!!----
!!---- Program: TestHall_DB
!!----
!!---- JGP May2019, JRC Aug2020
!!
Program Test_MHall_DB
   !---- Use Modules ----!
   Use CFML_Globaldeps, only: cp, err_CFML,clear_error
   Use CFML_Strings,    only: pack_string
   Use CFML_Symmetry_Tables
   Use CFML_Magnetic_Database
   Use CFML_gSpaceGroups

   !---- Variables ----!
   implicit none

   Type :: MAG_SYMB_TYPE
      character(len=40) :: BNS
      character(len=40) :: OG
      character(len=40) :: STD
      character(len=40) :: HallM
      character(len=15) :: ID_BNS
      character(len=15) :: ID_OG
   End Type MAG_SYMB_TYPE
   ! Type, public :: Shub_Spgr_Info_Type
   !    character(len=8)  :: ID_BNS =" "   !  ID Number of BNS
   !    character(len=15) :: BNS    =" "   !  BNS symbol
   !    character(len=12) :: ID_OG  =" "   !  ID number of OG
   !    character(len=15) :: OG     =" "   !  OG symbol
   !    character(len=25) :: STD    =" "   !  Proposed standard symbol
   !    character(len=25) :: MHall  =" "   !  Magnetic Hall symbol
   ! End Type Shub_Spgr_Info_Type

   type(Mag_Symb_Type), dimension(1651) :: Mag_Symb

   character(len=50)                            :: str_BNS, str_Hall
   character(len=60), dimension(:), allocatable :: gen
   character(len=:), allocatable                :: generators
   integer                                      :: i,n, k, j,nt, i_out=1
   integer                                      :: ik,ngen
   real(kind=cp), dimension(3)                  :: shift
   type(spg_type)                               :: SpG
   real(kind=cp) :: start, fin, st,fi
   !integer, dimension(1651)                     :: Litvin_to_IT

   !> Magnetic Symmetry Symbols (Proposed)
   call set_shubnikov_info()
   Mag_symb(:)%BNS=shubnikov_info(:)%bns
   Mag_symb(:)%HallM=shubnikov_info(:)%MHall

   !> Magnetic database from FullProf
   call Read_Magnetic_Data()
   if (err_CFML%IErr /= 0) then
      write(*,"(a)") trim(err_cfml%msg)
      stop
   end if

   call system("cls")
   write(*,"(a)")'-----------------------------------------------------'
   write(*,"(a)")' Testing Magnetic Symmetry Groups using Hall symbols'
   write(*,"(a)")'-----------------------------------------------------'

   !> Main
   call cpu_time(start)
   nt=0
   open(unit=i_out,file="Magnetic_Hall_Symbols.txt",status="replace",action="write")
   write(i_out,"(a)")&
   'Litvin_Number  BNS_number   BNS_symbol         OG_number        OG_symbol      INT_number   INT_symbol                  MHall_symbol                    Explicit Generators separated by ";"'
   do n=1,1651
      !> Info from Database
      call cpu_time(st)
      ik=Litvin2IT(n)
      !str_BNS=adjustl(spacegroup_label_bns(n))

      !> Magnetic Hall symbol
      str_Hall=trim(Mag_symb(ik)%HallM)
      ngen=0
      gen=" "
      call Get_Generators(str_Hall, Gen, ngen)
      generators=" "
      do i=1,ngen
        generators=trim(generators)//trim(gen(i))//";"
      end do
      generators=generators(1:len_trim(generators)-1)
      !
      !!> Constructor
      call Init_SpaceGroup(SpG)
      call Group_Constructor(gen,SpG)
      if (Err_CFML%Ierr /= 0) then
         write(*,"(a)") '    --->'//trim(Err_CFML%Msg)
         cycle
      end if
      !
      !!> Identify group
      call Identify_Group(Spg)
      if(Err_CFML%Ierr == 1) then
         write(unit=*,fmt="(a)") "  WARNING: "//Err_CFML%Msg
         call clear_error()
      end if
      if(len_trim(SpG%BNS_num) /= 0 .and. SpG%numshu /= 0 .and. len_trim(SpG%BNS_symb) == 0) then
        call set_Shubnikov_info()
        SpG%BNS_symb=Shubnikov_Info(Litvin2IT(SpG%numshu))%BNS
      end if
      call cpu_time(fi)
      !
      str_BNS=SpG%BNS_symb  !here we obtain the symbol generated by Identify_Group
      nt=nt+1
      !Litvin_to_IT(n)=ik
      if (trim(str_BNS) /= trim(Mag_symb(ik)%BNS)) then
         write(unit=*,fmt='(i6,i6,t18,a,t40,a,t60,a,f8.4,a)') nt, ik, str_BNS, Mag_symb(ik)%BNS, 'ERROR  '//shubnikov_info(ik)%STD//shubnikov_info(ik)%MHALL//'  CPU:',fi-st," seconds"
      else
         call Get_Generators(str_Hall, gen, ngen)
         write(unit=*,fmt='(i6,i6,t18,a,t40,a,t60,a,f8.4,a)') nt, ik, str_BNS, Mag_symb(ik)%BNS, 'OK     '//shubnikov_info(ik)%STD//shubnikov_info(ik)%MHALL//'  CPU:',fi-st," seconds  => "//trim(generators)
         write(unit=i_out,fmt='(i6,tr6,4(tr5,a),i6,tr5,a,tr5,a)')  &
            ik,shubnikov_info(ik)%id_BNS, shubnikov_info(ik)%BNS,shubnikov_info(ik)%ID_OG,shubnikov_info(ik)%OG, nt, shubnikov_info(ik)%STD,shubnikov_info(ik)%MHall//"    "//trim(generators)
      end if

   end do
   call cpu_time(fin)
   write(*,"(a,f12.3,a)") "Total CPU_TIME for this calculation: ",(fin-start)/60," minutes"
   !open(unit=1,file="Litvin_to_IT.txt",action="write",status="replace")
   !write(1,"(a)")  "Litvin to International pointer "
   !write(1,"(20(i5,a))") (Litvin_to_IT(n),",",n=1,1651)
   !close(unit=1)
End Program Test_MHall_DB
